#
# Revelation 0.1.0 - a password manager for GNOME 2
# http://oss.wired-networks.net/revelation/
#
# Module containing dialog classes
#
#
# Copyright (c) 2003-2004 Erik Grinaker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import gtk, gnome, gnome.ui, revelation, time

# first we define a few base classes

class Hig(gtk.Dialog):
	"""Displays a HIGified dialog"""

	def __init__(self, parent, pritext, sectext, stockimage, buttons, default = None):

		gtk.Dialog.__init__(self, "", parent, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL)

		self.set_border_width(6)
		self.set_has_separator(gtk.FALSE)
		self.set_resizable(gtk.FALSE)
		self.vbox.set_spacing(12)

		self.__init_contents(pritext, sectext, stockimage)
		self.__init_buttons(buttons, default)


	def __init_buttons(self, buttons, default):
		"""Sets up the buttons for the dialog"""

		for button, i in zip(buttons, range(len(buttons))):
			b = self.add_button(button[0], button[1])

			# set alternate text, if defined
			if len(button) == 3:
				b.get_children()[0].get_children()[0].get_children()[1].set_label(button[2])

			# set default button
			if default == i or (default == None and i == 0):
				self.set_default_response(button[1])


	def __init_contents(self, pritext, sectext, stockimage):
		"""Sets up the content area of the dialog"""

		# hbox separating dialog image and contents
		hbox = gtk.HBox()
		hbox.set_spacing(12)
		hbox.set_border_width(6)
		self.vbox.pack_start(hbox)

		# set up image
		image = gtk.Image()
		image.set_from_stock(stockimage, gtk.ICON_SIZE_DIALOG)
		image.set_alignment(0.5, 0)
		hbox.pack_start(image)

		# set up main content area
		self.contents = gtk.VBox()
		self.contents.set_spacing(10)
		hbox.pack_start(self.contents)

		# set contents
		label = gtk.Label("<span size=\"larger\" weight=\"bold\">" + pritext + "</span>\n\n" + sectext)
		label.set_use_markup(gtk.TRUE)
		label.set_line_wrap(gtk.TRUE)
		label.set_alignment(0, 0)
		self.contents.pack_start(label)


	def run(self):
		"""Runs the dialog, and returns the response"""

		self.show_all()
		response = gtk.Dialog.run(self)
		self.destroy()

		return response


class Property(gtk.Dialog):

	def __init__(self, parent, title, buttons, default = None):

		gtk.Dialog.__init__(self, title, parent, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL)

		self.set_border_width(12)
		self.set_has_separator(gtk.FALSE)
		self.set_resizable(gtk.FALSE)
		self.vbox.set_spacing(18)

		self.__init_buttons(buttons, default)

		self.sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)


	def __init_buttons(self, buttons, default):
		"""Sets up the dialog buttons"""

		for button, i in zip(buttons, range(len(buttons))):
			b = self.add_button(button[0], button[1])

			# set alternate text, if defined
			if len(button) == 3:
				b.get_children()[0].get_children()[0].get_children()[1].set_label(button[2])

			# set default
			if default == i or (default == None and i == 0):
				self.set_default_response(button[1])


	def add_section(self, title):
		"""Adds an inputsection to the dialog"""

		section = revelation.ui.InputSection(title, self.sizegroup)
		self.vbox.pack_start(section)

		return section



# the following classes may be subclassed from the bases above

class About(gnome.ui.About):
	"""Displays an about dialog"""

	def __init__(self):

		gnome.ui.About.__init__(
			self, revelation.APPNAME, revelation.VERSION, revelation.COPYRIGHT,
			"\"" + revelation.RELNAME + "\"\n\nRevelation is a password manager for the GNOME 2 desktop.",
			[ revelation.AUTHOR ], None, "",
			gtk.gdk.pixbuf_new_from_file(revelation.DATADIR + "/pixmaps/revelation.png")
		)


	def run(self):
		self.show_all()


class EditEntry(Property):
	"""Dialog for editing account entries"""

	def __init__(self, parent, title, entrytypes):

		Property.__init__(self, parent, title, [ [ gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ], [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ], 1)

		# set a few properties
		self.entrytypes = entrytypes
		self.fieldsection = None

		# get list of types, place folder type as first item
		self.typelist = self.entrytypes.get_list()
		self.typelist.remove(revelation.data.ENTRY_FOLDER)
		self.typelist.insert(0, revelation.data.ENTRY_FOLDER)

		# set up meta section
		self.__init_metasection(title)

		# set default type to folder
		self.update(revelation.data.ENTRY_FOLDER)


	def __init_metasection(self, title):
		"""Sets up the meta section"""

		self.metasection = self.add_section(title)

		self.entry_name = self.ui_textentry()
		self.metasection.add_inputrow("Name", self.entry_name)

		self.entry_desc = self.ui_textentry()
		self.metasection.add_inputrow("Description", self.entry_desc)

		# set up the dropdown menu
		menu = gtk.Menu()

		for type in self.typelist:

			item = gtk.ImageMenuItem()
			menu.append(item)

			image = gtk.Image()
			image.set_from_stock(self.entrytypes.get_data(type)["icon"], revelation.ui.ICON_SIZE_DROPDOWN)
			item.set_image(image)

			label = gtk.Label(self.entrytypes.get_data(type)["name"])
			label.set_alignment(0, 0.5)
			item.add(label)

			# put separator after folder item
			if type == revelation.data.ENTRY_FOLDER:
				menu.append(gtk.SeparatorMenuItem())

		self.dropdown = gtk.OptionMenu()
		self.dropdown.set_menu(menu)
		self.dropdown.connect("changed", self.cb_update, None)
		self.metasection.add_inputrow("Type", self.dropdown)


	def ui_textentry(self, value = None):
		"""Creates a entry widget for normal text input"""

		entry = gtk.Entry()
		entry.set_width_chars(50)
		entry.set_activates_default(gtk.TRUE)

		if value != None:
			entry.set_text(value)

		return entry


	def cb_update(self, object, data):
		"""Callback for updating the field form"""

		# only update field form if type has changed
		type = self.get_type()
		if self.curtype != type:
			self.update(type)
			self.show_all()


	def get_type(self):
		"""Fetches the currently selected type from the dropdown"""

		typeindex = self.dropdown.get_history()

		# remove 1 because of separator
		if typeindex > 0:
			typeindex = typeindex - 1

		return self.typelist[typeindex]


	def get_data(self):
		"""Returns the current data from the dialog"""

		data = {
			"name"		: self.entry_name.get_text(),
			"description"	: self.entry_desc.get_text(),
			"type"		: self.get_type(),
			"icon"		: self.entrytypes.get_data(self.get_type())["icon"],
			"updated"	: time.time(),
			"fields"	: {}
		}

		for field in self.entrytypes.get_data(data["type"])["fields"]:
			data["fields"][field] = self.entry_field[field].get_text()

		return data


	def run(self):
		"""Displays the dialog, and returns the data"""

		self.show_all()

		# run dialog and fetch data
		if Property.run(self) == gtk.RESPONSE_OK:
			data = self.get_data()

			# check if name is filled in
			if data["name"] == "":
				Error(self, "No name given", "You need to enter a name for the entry.").run()
				return self.run()

		else:
			data = None

		self.destroy()

		return data


	def set_data(self, data):
		"""Updates the dialog with data"""

		self.entry_name.set_text(data["name"])
		self.entry_desc.set_text(data["description"])
		self.update(data["type"], data["fields"])


	def set_type(self, type):
		"""Sets the current type for the dialog"""

		# get type index
		for t, i in zip(self.typelist, range(len(self.typelist))):
			if type == t:
				typeindex = i

		# add one if not first item, because of separator
		if typeindex > 0:
			typeindex = typeindex + 1

		# set type if not already selected
		if self.dropdown.get_history() != typeindex:
			self.dropdown.set_history(typeindex)

		# store current type
		self.curtype = type


	def set_typechange_allowed(self, allow):
		"""Sets whether it is allowed to change the type"""

		# get menu items, without separator
		items = self.dropdown.get_menu().get_children()
		del items[1]

		for type, item in zip(self.typelist, items):
			if allow == gtk.FALSE and type != self.curtype:
				item.set_sensitive(gtk.FALSE)
			else:
				item.set_sensitive(gtk.TRUE)


	def update(self, type, fielddata = None):
		"""Updates the field form for the selected type"""

		self.set_type(type)
		fields = self.entrytypes.get_data(type)["fields"]

		# set a few vars
		self.entry_field = {}

		# don't set up field section if not needed
		if len(fields) == 0:

			# remove field section if it exists
			if self.fieldsection != None:
				self.fieldsection.destroy()
				self.fieldsection = None

		else:

			# create field form if it doesn't exist
			if self.fieldsection == None:
				self.fieldsection = self.add_section("Account data")

			# empty the field form
			self.fieldsection.clear()

			# set up fields
			for field in fields:

				if fielddata != None and fielddata.has_key(field):
					value = fielddata[field]
				else:
					value = ""

				self.entry_field[field] = self.ui_textentry(value)
				self.fieldsection.add_inputrow(self.entrytypes.get_field_data(field)["name"], self.entry_field[field])


class Error(Hig):

	def __init__(self, parent, pritext, sectext):
		Hig.__init__(
			self, parent, pritext, sectext, gtk.STOCK_DIALOG_ERROR,
			[ [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ]
		)


class FileSelector(gtk.FileSelection):

	def __init__(self, title = None):
		gtk.FileSelection.__init__(self, title)


	def run(self):
		"""displays the file selector"""

		self.show()

		if gtk.FileSelection.run(self) == gtk.RESPONSE_OK:
			filename = self.get_filename()
		else:
			filename = None

		self.destroy()

		return filename


class FolderSelection(Property):
	"""Dialog with a tree of folders"""

	def __init__(self, parent, title, datastore, default = None):

		Property.__init__(self, parent, "", [ [ gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ], [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ])

		self.set_resizable(gtk.TRUE)
		self.set_default_size(250, 300)

		section = self.add_section(title)
		self.folders = revelation.data.FolderStore(datastore)

		# set up the tree
		scrolledwindow = gtk.ScrolledWindow()
		scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
		section.pack_start(scrolledwindow)

		self.tree = revelation.ui.Tree(self.folders, "Folders")
		scrolledwindow.add(self.tree)

		# find the default iter in the folderstore
		self.default = self.folders.get_iter_from_orig(default)


	def run(self):

		self.show_all()

		# select the default
		if self.default == None:
			self.tree.unselect_all()
		else:
			self.tree.select(self.default)

		# run the dialog
		if Property.run(self) == gtk.RESPONSE_OK:
			iter = self.tree.get_selected()

			if iter != None:
				iter = self.folders.get_orig_iter(iter)
		else:
			iter = None

		self.destroy()

		return iter


class Password(Hig):
	"""Displays a dialog asking for a password"""

	def __init__(self, parent, title, text, confirm = gtk.FALSE, current = None):

		Hig.__init__(
			self, parent, title, text, revelation.ui.STOCK_PASSWORD,
			[ [ gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ], [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ], 1
		)

		self.confirm = confirm
		self.current = current

		inputsection = revelation.ui.InputSection()
		self.contents.pack_start(inputsection)

		# ask for current password
		if current != None:
			self.entry_current = self.ui_passwordentry()
			inputsection.add_inputrow("Current", self.entry_current)

		# password entry
		self.entry_password = self.ui_passwordentry()
		inputsection.add_inputrow("Password", self.entry_password)

		# ask for password confirmation
		if confirm == gtk.TRUE:
			self.entry_confirm = self.ui_passwordentry()
			inputsection.add_inputrow("Confirm", self.entry_confirm)


	def ui_passwordentry(self):
		"""Creates a password entry field"""

		entry = gtk.Entry()
		entry.set_visibility(gtk.FALSE)
		entry.set_activates_default(gtk.TRUE)

		return entry


	def run(self):

		self.show_all()

		# get password
		if Property.run(self) == gtk.RESPONSE_OK:
			password = self.entry_password.get_text()

			# check if current password is correct
			if self.current != None and self.current != self.entry_current.get_text():
				Error(self, "Incorrect password", "The password you entered as the current file password is incorrect.").run()
				return self.run()

			# display error if no password
			if len(password) == 0:
				Error(self, "Password not given", "You need to enter a password.").run()
				return self.run()

			# check if confirm password matches
			if self.confirm == gtk.TRUE and password != self.entry_confirm.get_text():
				Error(self, "Passwords don't match", "The password and password confirmation you entered does not match.").run()
				return self.run()

		else:
			password = None

		self.destroy()

		return password


class RemoveEntry(Hig):
	"""Displays a dialog to confirm entry deletion"""

	def __init__(self, parent, type, name):

		if type == revelation.data.ENTRY_FOLDER:
			title = "Really remove folder '" + name + "'?"
			text = "By removing this folder you will also remove all accounts and folders it contains. You will not be able to recover these."
		else:
			title = "Really remove account '" + name + "'?"
			text = "You will not be able to recover the data for this account after it has been removed."

		Hig.__init__(
			self, parent, title, text, gtk.STOCK_DIALOG_WARNING,
			[ [ gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ], [ gtk.STOCK_DELETE, gtk.RESPONSE_OK, "Remove" ] ]
		)


	def run(self):
		"""Run the dialog"""

		if Hig.run(self) == gtk.RESPONSE_OK:
			return gtk.TRUE
		else:
			return gtk.FALSE

